/*
 * Copyright (C) 2005-2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file recv.h receiver-relates structures and exported symbols of recv_common.c */

/**
 * describes one instance of a receiver
 */
struct receiver_node {
	/** points to the corresponding receiver */
	struct receiver *receiver;
	/** the output buffer */
	char *buf;
	/** the amount of bytes in \a buf */
	size_t loaded;
	/** receiver-specific data */
	void *private_data;
	/** set to 1 if end of file is reached */
	int eof;
	/** pointer to the configuration data for this instance */
	void *conf;
};

/**
 * describes one supported paraslash receiver
 *
 * \sa http_recv.c, ortp_recv.c
 */
struct receiver {
/**
 *
 *
 * the name of the receiver
 */
	const char *name;
/**
 *
 *
 * the receiver init function
 *
 * It must fill in all other function pointers and is assumed to succeed.
 *
 * \sa http_recv_init ortp_recv_init
 */
	void (*init)(struct receiver *r);
/**
 *
 *
 * the command line parser of the receiver
 *
 * It should check whether the command line options given by \a argc and \a
 * argv are valid.  On success, it should return a pointer to the
 * receiver-specific configuration data determined by \a argc and \a argv.
 * Note that this might be called more than once with different values of
 * \a argc and \a argv.
 *
 */
	void * (*parse_config)(int argc, char **argv);
/**
 *
 *
 * open one instance of the receiver
 *
 * This should allocate the output buffer of \a rn. and may also perform any
 * other work necessary for retrieving the stream according to the
 * configuration stored in the \a conf member of \a rn which is guaranteed to
 * point to valid configuration data (as previously obtained from the config
 * parser).
 *
 * \sa receiver_node::conf, receiver_node::buf
 */
	int (*open)(struct receiver_node *rn);
/**
 *
 *
 * close one instance of the receiver
 *
 * It should free all resources associated with given receiver node that were
 * allocated during the corresponding open call.
 *
 * \sa receiver_node
 */
	void (*close)(struct receiver_node *rn);
/**
 *
 *
 * deactivate the receiver
 *
 * Clean up what init has allocated.
 */
	void (*shutdown)(void);
/**
 *
 *
 * add file descriptors to fd_sets and compute timeout for select(2)
 *
 * The pre_select function gets called from the driving application before
 * entering its select loop. The receiver may use this hook to add any file
 * descriptors to \a rfds and \a wfds in order to check the result later in the
 * post_select hook.
 *
 * \a timeout is a value-result parameter, initially containing the timeout for
 * select() which was set by the application or by another receiver node. If
 * the receiver wants its pre_select function to be called at some earlier time
 * than the time determined by \a timeout, it may set \a timeout to an
 * appropriate smaller value. However, it must never increase this timeout.
 *
 * This function must return the highest-numbered descriptor it wants to being
 * checked, or -1 if no file descriptors should be checked for this run.
 *
 * \sa select(2), receiver_node:private_data, time.c
 */
	int (*pre_select)(struct receiver_node *rn, fd_set *rfds,
		fd_set *wfds, struct timeval *timeout);
/**
 *
 *
 * evaluate the result from select()
 *
 * If the call to select() was succesful, this hook gets called. It should
 * check all file descriptors which were added to any of the the fd sets during
 * the previous call to pre_select. According to the result, it may then use
 * any non-blocking I/O to establish a connection or to receive the audio data.
 *
 * A negative return value is interpreted as an error.
 *
 * \sa select(2), struct receiver
 */
	int (*post_select)(struct receiver_node *rn, int select_ret,
		fd_set *rfds, fd_set *wfds);
};


/** \cond */
extern void http_recv_init(struct receiver *r);
#define HTTP_RECEIVER {.name = "http", .init = http_recv_init},
extern void dccp_recv_init(struct receiver *r);
#define DCCP_RECEIVER {.name = "dccp", .init = dccp_recv_init},

#ifdef HAVE_ORTP
extern void ortp_recv_init(struct receiver *r);
#define ORTP_RECEIVER {.name = "ortp", .init = ortp_recv_init},
#else
#define ORTP_RECEIVER
#endif

void *check_receiver_arg(char *ra, int *receiver_num);


extern struct receiver receivers[];
extern void (*crypt_function_recv)(unsigned long len, const unsigned char *indata, unsigned char *outdata);
extern void (*crypt_function_send)(unsigned long len, const unsigned char *indata, unsigned char *outdata);

#define DEFINE_RECEIVER_ARRAY struct receiver receivers[] = { \
	HTTP_RECEIVER \
	DCCP_RECEIVER \
	ORTP_RECEIVER \
	{.name = NULL}};

/** \endcond */
